/* This file is part of the KDE project
   Copyright (C) 2015 Jarosław Staniek <staniek@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef KREPORTSTATICPLUGININTERFACE_H
#define KREPORTSTATICPLUGININTERFACE_H

#define QT_STATICPLUGIN

#include "KReportPluginInterface.h"

//! Declaration to put in all static plugin headers
#define KREPORT_DECLARE_STATIC_PLUGIN(name) \
    extern const QStaticPlugin qt_static_plugin_ ## name ## Factory(); \

//! Used by manager to access metadata
#define KREPORT_STATIC_PLUGIN_METADATA(name) \
    qt_static_plugin_ ## name ## Factory().metaData()

#endif // KREPORTSTATICPLUGININTERFACE_H
